/*
 * @(#)FastCharsetProvider.java        1.3        05/11/17
 *
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 *
 * Contributors: Ulf Zibis, Cologne <Ulf.Zibis @ CoSoCo.de>
 */

package sun.nio.cs;

import java.nio.charset.Charset;
import java.nio.charset.spi.CharsetProvider;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * Abstract base class for fast charset providers.
 *
 * @author Mark Reinhold
 * @author Ulf Zibis, Cologne <Ulf.Zibis @ CoSoCo.de>
 * @version 1.3, 05/11/17
 */
public abstract class FastCharsetProvider extends CharsetProvider {

    final String packagePrefix = getClass().getPackage().getName();

    // Caches charset instances for iterator from charsets()
    protected final Set<Charset> charsets;

    // Maps uppercased canonical and alias names to charset instances
    final Map<String,Charset> cache;

    FastCharsetProvider(int length, Map<String,Charset> cache) {
        charsets = new HashSet(length);
        this.cache = cache;
    }

    public final Charset charsetForName(String charsetName) {
        return lookup(toUpper(charsetName));
    }

    public final Iterator<Charset> charsets() {
        return Collections.unmodifiableSet(charsets0()).iterator();
    }

    synchronized Set<Charset> charsets0() {
        if (charsets.isEmpty())
            charsets.addAll(cache.values());
        return charsets;
    }

    Charset lookup(String uppercasedName) {
        return cache.get(uppercasedName);
    }

    // Private ASCII-only optimized version, for interpretation during startup
    public static final String toUpper(final String s) {
        boolean allUpper = true;
        char[] ca = null;
        for (int i=0; i<s.length(); i++) {
            int c = s.charAt(i);
            if (c >= 'a' && c <= 'z') {
                if (allUpper) {
                    ca = s.toCharArray();
                    allUpper = false;
                }
                ca[i] -= '\u0020';
            }
        }
        return allUpper ? s : new String(ca);
    }
}
