/*
 * @(#)Mapping.java        0.0        09/07/29
 *
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 *
 * Contributors: Ulf Zibis, Germany, Ulf.Zibis @ CoSoCo.de
 */

package sun.nio.cs;

import java.util.Arrays;

/**
 * Abstract class for classes holding charset mappings.
 *
 * @author Ulf Zibis, Cologne <Ulf.Zibis @ CoSoCo.de>
 */
//public abstract class Mapping<<M extends Mapping>,D extends FastCharset.Decoder<M>,E extends FastCharset.Encoder<M>> {
public abstract class Mapping<D extends FastCharset.Decoder,E extends FastCharset.Encoder> {
//public abstract class Mapping<D extends FastCharset.Decoder> {
//public abstract class Mapping<D extends FastCharset.Decoder<M>,M extends Mapping<D,M>> {

    public static final char UNMAPPABLE_DECODING = '\uFFFD';
    public static final int  UNMAPPABLE_ENCODING = 0xFFFD;

    public static final char MAX_ASCII = '\u007F';
    public static final char BEYOND_ASCII = MAX_ASCII + 1;
    public static final char BYTE_RANGE = 1 << Byte.SIZE;
    public static final char REPLACE_CHAR = '\uFFFD';
    public static final char NOT_A_CHAR = '\uFFFE';
    public static final char[] REPLACE_CHARS = new char[0x100];
    static { Arrays.fill(REPLACE_CHARS, REPLACE_CHAR); }

    final String historicalName;
    final boolean isASCII;

    // TODO: Load these mappings from separate data file.
    // This would reduce the footprint of the whole sun.nio.cs.* packages.
    // Then additionally there won't remain the need to have an extra
    // subclass from this class for each charset. This would remarkable
    // reduce the complexity of the sun.nio.cs.* packages.
    protected Mapping(String historicalName, boolean isASCII) {
        this.historicalName = historicalName;
        this.isASCII = isASCII;
    }

    protected abstract D newDecoder(FastCharset cs);

    protected abstract E newEncoder(FastCharset cs);
//    protected abstract <M extends Mapping> FastCharset.Encoder<M> newEncoder(FastCharset cs);
//    protected abstract FastCharset.Encoder<M> newEncoder(FastCharset cs);
}
