/*
 * @(#)SingleByteMapping.java        0.0        09/07/29
 *
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 *
 * Contributors: Ulf Zibis, Germany, Ulf.Zibis @ CoSoCo.de
 */

package sun.nio.cs;

import java.nio.charset.Charset;

/**
 * Abstract class for classes holding single-byte charset mappings.
 *
 * @author Ulf Zibis, Cologne <Ulf.Zibis @ CoSoCo.de>
 */
public abstract class SingleByteMapping extends Mapping<SingleByte.Decoder,SingleByte.Encoder> {

    // TODO: Load these mappings from separate data file.
    // This would reduce the footprint of the whole sun.nio.cs.* packages.
    // Then additionally there won't remain the need to have an extra
    // subclass from this class for each charset. This would remarkable
    // reduce the complexity of the sun.nio.cs.* packages.
    // In case the accessors to sun.io converters are no more needed, then
    // instead of a char[] c2b should be better a byte[] and b2cTable could
    // be dropped, otherwise create b2cTable by new String(b2c).
    private final String b2cTable;
    final char[] b2c;
    final int c2bLength;
    final char[] nrBytes;
    final char[] c2bChars;
    final char[] c2bBytes;

    protected SingleByteMapping(String historicalName, boolean isASCII,
            String b2cTable, int c2bLength, char[] nrBytes, char[] c2bChars, char[] c2bBytes) {
        super (historicalName, isASCII);
        this.b2cTable = b2cTable;
        this.b2c = b2cTable.toCharArray();
        this.c2bLength = c2bLength;
        this.nrBytes = nrBytes;
        this.c2bChars = c2bChars;
        this.c2bBytes = c2bBytes;
    }

    protected SingleByte.Decoder newDecoder(FastCharset cs) {
        return new SingleByte.Decoder(cs, this);
    }

    protected SingleByte.Encoder newEncoder(FastCharset cs) {
        return new SingleByte.Encoder(cs, this);
    }


    /**
     * These accessors are temporarily supplied while sun.io converters co-exist
     * with the sun.nio.cs.{ext} charset coders.  These facilitate sharing of
     * conversion tables between the two co-existing implementations.
     * TODO: When sun.io converters are made extinct or purged to smart adapter
     * class, these will be unncessary and should be removed
     */
    public String getDecoderSingleByteMappings() {
        return b2cTable;
    }

    public char[] getEncoderIndex1() {
        return ((SingleByte.Encoder)Charset.forName(historicalName).newEncoder()).c2bIndex;
    }

    public char[] getEncoderIndex2() {
        return ((SingleByte.Encoder)Charset.forName(historicalName).newEncoder()).c2b;
    }
}
